@@ -462,6 +462,11 @@ act_stage2_config (NMDevice *device, NMDeviceStateReason *out_failure_reason)
 
 	nm_clear_g_source (&priv->sup_timeout_id);
 
+	if (!priv->mgmt_iface) {
+		NM_SET_OUT (out_failure_reason, NM_DEVICE_STATE_REASON_SUPPLICANT_FAILED);
+		return NM_ACT_STAGE_RETURN_FAILURE;
+	}
+
 	connection = nm_device_get_applied_connection (device);
 	g_return_val_if_fail (connection, NM_ACT_STAGE_RETURN_FAILURE);
 
@@ -583,50 +588,30 @@ remove_all_peers (NMDeviceWifiP2P *self)
 
 
 static NMActStageReturn
-act_stage3_ip4_config_start (NMDevice *device,
-                             NMIP4Config **out_config,
-                             NMDeviceStateReason *out_failure_reason)
+act_stage3_ip_config_start (NMDevice *device,
+                            int addr_family,
+                            gpointer *out_config,
+                            NMDeviceStateReason *out_failure_reason)
 {
+	gboolean indicate_addressing_running;
 	NMConnection *connection;
-	NMSettingIPConfig *s_ip4;
-	const char *method = NM_SETTING_IP4_CONFIG_METHOD_AUTO;
+	const char *method;
 
 	connection = nm_device_get_applied_connection (device);
-	g_return_val_if_fail (connection, NM_ACT_STAGE_RETURN_FAILURE);
 
-	s_ip4 = nm_connection_get_setting_ip4_config (connection);
-	if (s_ip4)
-		method = nm_setting_ip_config_get_method (s_ip4);
+	method = nm_utils_get_ip_config_method (connection, addr_family);
 
-	/* Indicate that a critical protocol is about to start */
-	if (nm_streq (method, NM_SETTING_IP4_CONFIG_METHOD_AUTO))
+	if (addr_family == AF_INET)
+		indicate_addressing_running = NM_IN_STRSET (method, NM_SETTING_IP4_CONFIG_METHOD_AUTO);
+	else {
+		indicate_addressing_running = NM_IN_STRSET (method, NM_SETTING_IP6_CONFIG_METHOD_AUTO,
+		                                                    NM_SETTING_IP6_CONFIG_METHOD_DHCP);
+	}
+
+	if (indicate_addressing_running)
 		nm_platform_wifi_indicate_addressing_running (nm_device_get_platform (device), nm_device_get_ip_ifindex (device), TRUE);
 
-	return NM_DEVICE_CLASS (nm_device_wifi_p2p_parent_class)->act_stage3_ip4_config_start (device, out_config, out_failure_reason);
-}
-
-static NMActStageReturn
-act_stage3_ip6_config_start (NMDevice *device,
-                             NMIP6Config **out_config,
-                             NMDeviceStateReason *out_failure_reason)
-{
-	NMConnection *connection;
-	NMSettingIPConfig *s_ip6;
-	const char *method = NM_SETTING_IP6_CONFIG_METHOD_AUTO;
-
-	connection = nm_device_get_applied_connection (device);
-	g_return_val_if_fail (connection, NM_ACT_STAGE_RETURN_FAILURE);
-
-	s_ip6 = nm_connection_get_setting_ip6_config (connection);
-	if (s_ip6)
-		method = nm_setting_ip_config_get_method (s_ip6);
-
-	/* Indicate that a critical protocol is about to start */
-	if (NM_IN_STRSET (method, NM_SETTING_IP6_CONFIG_METHOD_AUTO
-	                          NM_SETTING_IP6_CONFIG_METHOD_DHCP))
-		nm_platform_wifi_indicate_addressing_running (nm_device_get_platform (device), nm_device_get_ip_ifindex (device), TRUE);
-
-	return NM_DEVICE_CLASS (nm_device_wifi_p2p_parent_class)->act_stage3_ip6_config_start (device, out_config, out_failure_reason);
+	return NM_DEVICE_CLASS (nm_device_wifi_p2p_parent_class)->act_stage3_ip_config_start (device, addr_family, out_config, out_failure_reason);
 }
 
 static void
@@ -705,11 +690,10 @@ supplicant_iface_state_cb (NMSupplicantInterface *iface,
 			_set_is_waiting_for_supplicant (self, FALSE);
 		break;
 	case NM_SUPPLICANT_INTERFACE_STATE_DOWN:
+		supplicant_interfaces_release (self, TRUE);
 		nm_device_queue_recheck_available (device,
 		                                   NM_DEVICE_STATE_REASON_SUPPLICANT_AVAILABLE,
 		                                   NM_DEVICE_STATE_REASON_SUPPLICANT_FAILED);
-
-		supplicant_interfaces_release (self, TRUE);
 		break;
 	default:
 		break;
@@ -931,20 +915,14 @@ supplicant_interfaces_release (NMDeviceWifiP2P *self, gboolean set_is_waiting)
 
 	if (priv->mgmt_iface) {
 		_LOGD (LOGD_DEVICE | LOGD_WIFI, "P2P: Releasing WPA supplicant interface.");
-
 		nm_supplicant_manager_set_wfd_ies (priv->sup_mgr, NULL);
-
 		g_signal_handlers_disconnect_by_data (priv->mgmt_iface, self);
-
 		g_clear_object (&priv->mgmt_iface);
+		nm_clear_g_source (&priv->sup_timeout_id);
 	}
 
 	supplicant_group_interface_release (self);
 
-	nm_device_state_changed (NM_DEVICE (self),
-	                         NM_DEVICE_STATE_UNAVAILABLE,
-	                         NM_DEVICE_STATE_REASON_SUPPLICANT_FAILED);
-
 	if (set_is_waiting)
 		_set_is_waiting_for_supplicant (self, TRUE);
 }
@@ -970,8 +948,7 @@ device_state_changed (NMDevice *device,
 			supplicant_interfaces_release (self, TRUE);
 
 		/* TODO: More cleanup needed? */
-	} else
-		nm_assert (priv->mgmt_iface != NULL);
+	}
 
 	switch (new_state) {
 	case NM_DEVICE_STATE_UNMANAGED:
@@ -1084,6 +1061,14 @@ impl_device_wifi_p2p_stop_find (NMDBusObject *obj,
 	NMDeviceWifiP2P *self = NM_DEVICE_WIFI_P2P (obj);
 	NMDeviceWifiP2PPrivate *priv = NM_DEVICE_WIFI_P2P_GET_PRIVATE (self);
 
+	if (!priv->mgmt_iface) {
+		g_dbus_method_invocation_return_error_literal (invocation,
+		                                               NM_DEVICE_ERROR,
+		                                               NM_DEVICE_ERROR_NOT_ACTIVE,
+		                                               "WPA Supplicant management interface is currently unavailable.");
+		return;
+	}
+
 	nm_supplicant_interface_p2p_stop_find (priv->mgmt_iface);
 
 	g_dbus_method_invocation_return_value (invocation, NULL);
@@ -1118,7 +1103,8 @@ nm_device_wifi_p2p_set_mgmt_iface (NMDeviceWifiP2P *self,
 	if (!iface)
 		goto done;
 
-	_LOGD (LOGD_DEVICE | LOGD_WIFI, "P2P: WPA supplicant management interface changed to %s.", nm_supplicant_interface_get_object_path (iface));
+	_LOGD (LOGD_DEVICE | LOGD_WIFI, "P2P: WPA supplicant management interface changed to %s.",
+	       nm_supplicant_interface_get_object_path (iface));
 
 	priv->mgmt_iface = g_object_ref (iface);
 
@@ -1134,12 +1120,10 @@ nm_device_wifi_p2p_set_mgmt_iface (NMDeviceWifiP2P *self,
 	g_signal_connect (priv->mgmt_iface, NM_SUPPLICANT_INTERFACE_GROUP_STARTED,
 	                  G_CALLBACK (supplicant_iface_group_started_cb),
 	                  self);
-
+done:
 	nm_device_queue_recheck_available (NM_DEVICE (self),
 	                                   NM_DEVICE_STATE_REASON_SUPPLICANT_AVAILABLE,
 	                                   NM_DEVICE_STATE_REASON_SUPPLICANT_FAILED);
-
-done:
 	_set_is_waiting_for_supplicant (self,
 	                                   !priv->mgmt_iface
 	                                || (  nm_supplicant_interface_get_state (priv->mgmt_iface)
@@ -1154,6 +1138,14 @@ nm_device_wifi_p2p_remove (NMDeviceWifiP2P* self)
 
 /*****************************************************************************/
 
+static const char *
+get_type_description (NMDevice *device)
+{
+	return "wifi-p2p";
+}
+
+/*****************************************************************************/
+
 static const GDBusSignalInfo nm_signal_info_wifi_p2p_peer_added = NM_DEFINE_GDBUS_SIGNAL_INFO_INIT (
 	"PeerAdded",
 	.args = NM_DEFINE_GDBUS_ARG_INFOS (
@@ -1249,7 +1241,7 @@ nm_device_wifi_p2p_new (const char *iface)
 	return g_object_new (NM_TYPE_DEVICE_WIFI_P2P,
 	                     NM_DEVICE_IFACE, iface,
 	                     NM_DEVICE_TYPE_DESC, "802.11 Wi-Fi P2P",
-	                     NM_DEVICE_DEVICE_TYPE, NM_TYPE_DEVICE_WIFI_P2P,
+	                     NM_DEVICE_DEVICE_TYPE, NM_DEVICE_TYPE_WIFI_P2P,
 	                     NM_DEVICE_LINK_TYPE, NM_LINK_TYPE_WIFI,
 	                     NM_DEVICE_RFKILL_TYPE, RFKILL_TYPE_WLAN,
 	                     NULL);
@@ -1286,33 +1278,33 @@ nm_device_wifi_p2p_class_init (NMDeviceWifiP2PClass *klass)
 	NMDBusObjectClass *dbus_object_class = NM_DBUS_OBJECT_CLASS (klass);
 	NMDeviceClass *device_class = NM_DEVICE_CLASS (klass);
 
-	object_class->constructed = constructed;
+	object_class->constructed  = constructed;
 	object_class->get_property = get_property;
-	object_class->dispose = dispose;
-	object_class->finalize = finalize;
+	object_class->dispose      = dispose;
+	object_class->finalize     = finalize;
 
 	dbus_object_class->interface_infos = NM_DBUS_INTERFACE_INFOS (&interface_info_device_wifi_p2p);
 
-	device_class->connection_type_supported = NM_SETTING_WIFI_P2P_SETTING_NAME;
+	device_class->connection_type_supported        = NM_SETTING_WIFI_P2P_SETTING_NAME;
 	device_class->connection_type_check_compatible = NM_SETTING_WIFI_P2P_SETTING_NAME;
-	device_class->link_types = NM_DEVICE_DEFINE_LINK_TYPES (NM_LINK_TYPE_WIFI_P2P);
+	device_class->link_types                       = NM_DEVICE_DEFINE_LINK_TYPES (NM_LINK_TYPE_WIFI_P2P);
+	device_class->get_type_description             = get_type_description;
 
 	/* Do we need compatibility checking or is the default good enough? */
-	device_class->is_available = is_available;
-	device_class->check_connection_compatible = check_connection_compatible;
-	device_class->complete_connection = complete_connection;
+	device_class->is_available                     = is_available;
+	device_class->check_connection_compatible      = check_connection_compatible;
+	device_class->complete_connection              = complete_connection;
 
-	device_class->act_stage1_prepare = act_stage1_prepare;
-	device_class->act_stage2_config = act_stage2_config;
-	device_class->get_configured_mtu = get_configured_mtu;
-	device_class->get_auto_ip_config_method = get_auto_ip_config_method;
-	device_class->act_stage3_ip4_config_start = act_stage3_ip4_config_start;
-	device_class->act_stage3_ip6_config_start = act_stage3_ip6_config_start;
+	device_class->act_stage1_prepare               = act_stage1_prepare;
+	device_class->act_stage2_config                = act_stage2_config;
+	device_class->get_configured_mtu               = get_configured_mtu;
+	device_class->get_auto_ip_config_method        = get_auto_ip_config_method;
+	device_class->act_stage3_ip_config_start      = act_stage3_ip_config_start;
 
-	device_class->deactivate = deactivate;
-	device_class->unmanaged_on_quit = unmanaged_on_quit;
+	device_class->deactivate                       = deactivate;
+	device_class->unmanaged_on_quit                = unmanaged_on_quit;
 
-	device_class->state_changed = device_state_changed;
+	device_class->state_changed                    = device_state_changed;
 
 	obj_properties[PROP_PEERS] =
 	    g_param_spec_boxed (NM_DEVICE_WIFI_P2P_PEERS, "", "",
