@@ -34,7 +34,11 @@ else
     CALLED_FROM_MAKE=0
 fi
 
+BUILDDIR=
+
 if [ "$CALLED_FROM_MAKE" == 1 ]; then
+    BUILDDIR="$1"
+    shift
     if [ -n "$1" ]; then
         NMTST_LIBTOOL=($1 --mode=execute);
     else
@@ -144,6 +148,7 @@ else
             ;;
         esac
     done
+
     # we support calling the script directly. In this case,
     # only pass the path to the test to run.
     if test -z "${TEST+x}"; then
@@ -153,6 +158,20 @@ else
         NMTST_SUPPRESSIONS="$SCRIPT_PATH/../valgrind.suppressions"
     fi
 
+    if [[ -z "$NMTST_BUILDDIR" ]]; then
+        if [[ "${NMTST_BUILDDIR-x}" == x ]]; then
+            # autodetect
+            BUILDDIR="$(readlink -f "$TEST")"
+            while [[ -n "$BUILDDIR" ]]; do
+                BUILDDIR="$(dirname "$BUILDDIR")"
+                [[ "$BUILDDIR" == / ]] && BUILDDIR=
+                [[ -z "$BUILDDIR" ]] && break
+                [[ -e "$BUILDDIR/libnm/.libs/libnm.so" ]] && break
+                [[ -e "$BUILDDIR/libnm/libnm.so" ]] && break
+            done
+        fi
+    fi
+
 fi
 
 if [ "$NMTST_SET_DEBUG" == 1 -a -z "${NMTST_DEBUG+x}" ]; then
@@ -198,6 +217,16 @@ fi
 
 [ -x "$TEST" ] || die "Cannot execute test \"$TEST\""
 
+if [[ -n "$BUILDDIR" ]]; then
+    if [[ -d "$BUILDDIR/libnm" ]]; then
+        export GI_TYPELIB_PATH="$BUILDDIR/libnm/${GI_TYPELIB_PATH:+:$GI_TYPELIB_PATH}"
+        if [[ -d "$BUILDDIR/libnm/.libs" ]]; then
+            export LD_LIBRARY_PATH="$BUILDDIR/libnm/.libs${LD_LIBRARY_PATH:+:$LD_LIBRARY_PATH}"
+        else
+            export LD_LIBRARY_PATH="$BUILDDIR/libnm${LD_LIBRARY_PATH:+:$LD_LIBRARY_PATH}"
+        fi
+    fi
+fi
 
 if ! _is_true "$NMTST_USE_VALGRIND" 0; then
     "${NMTST_DBUS_RUN_SESSION[@]}" \
@@ -249,7 +278,7 @@ if [ $RESULT -ne 0 -a $RESULT -ne 77 ]; then
         UNRESOLVED=$(awk -F: '/obj:\// {print $NF}' "$LOGFILE" | sort | uniq)
         if [ -n "$UNRESOLVED" ]; then
             echo Some addresses could not be resolved into symbols. >&2
-            echo The errors might get suppressed when you install the debuging symbols. >&2
+            echo The errors might get suppressed when you install the debugging symbols. >&2
             if [ -x /usr/bin/dnf ]; then
                 echo Hint: dnf debuginfo-install $UNRESOLVED >&2
             elif [ -x /usr/bin/debuginfo-install ]; then
@@ -265,8 +294,11 @@ fi
 if [ $HAS_ERRORS -eq 0 ]; then
     # valgrind doesn't support setns syscall and spams the logfile.
     # hack around it...
-    if [ "$TEST_NAME" = 'test-link-linux' -a -z "$(sed -e '/^--[0-9]\+-- WARNING: unhandled .* syscall: /,/^--[0-9]\+-- it at http.*\.$/d' "$LOGFILE")" ]; then
-        HAS_ERRORS=1
+    if [ "$TEST_NAME" = 'test-link-linux' -o \
+         "$TEST_NAME" = 'test-acd' ]; then
+        if [ -z "$(sed -e '/^--[0-9]\+-- WARNING: unhandled .* syscall: /,/^--[0-9]\+-- it at http.*\.$/d' "$LOGFILE")" ]; then
+            HAS_ERRORS=1
+        fi
     fi
 fi
 
